@@ -1,13 +1,15 @@
 use super::config::*;
 use super::functions::*;
+use super::nums::*;
 use chrono::prelude::*;
 use std::collections::HashMap;
 use std::collections::HashSet;
+use std::ffi::OsString;
 use std::fs;
 use std::io::{Error, ErrorKind};
 use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus};
-use termion::{color, cursor, style};
+use termion::{clear, color, cursor, style};
 
 pub const STARTING_POINT: u16 = 3;
 pub const DOWN_ARROW: char = '\u{21D3}';
@@ -18,21 +20,23 @@ pub const TRASH: &str = "trash";
 pub const WHEN_EMPTY: &str = "Are you sure to empty the trash directory? (if yes: y)";
 
 macro_rules! print_item {
-    ($color: expr, $name: expr, $time: expr, $selected: expr, $time_start_pos: expr) => {
+    ($color: expr, $name: expr, $time: expr, $selected: expr, $time_start_pos: expr, $column: expr) => {
         if *($selected) {
             print!(
-                "{}{}{}{}{}{}{}",
+                "{}{}{}{}{}{} {}{}{}",
                 $color,
                 style::Invert,
                 $name,
+                style::Reset,
                 cursor::Left(60),
                 cursor::Right($time_start_pos),
+                style::Invert,
                 $time,
                 style::Reset
             );
         } else {
             print!(
-                "{}{}{}{}{}{}",
+                "{}{}{}{} {}{}",
                 $color,
                 $name,
                 cursor::Left(60),
@@ -41,6 +45,11 @@ macro_rules! print_item {
                 color::Fg(color::Reset)
             );
         }
+        if $column > 58 {
+            print!("{}", (0..8).map(|_| ' ').collect::<String>());
+        } else if $column > 49 {
+            print!("{}", (0..($column - 49)).map(|_| ' ').collect::<String>());
+        }
     };
 }
 #[derive(Clone)]
@@ -61,6 +70,8 @@ pub struct ItemInfo {
     pub file_type: FileType,
     pub file_name: String,
     pub file_path: std::path::PathBuf,
+    pub file_size: u64,
+    pub file_ext: Option<OsString>,
     pub modified: Option<String>,
     pub selected: bool,
 }
@@ -75,6 +86,7 @@ pub enum FileType {
 #[derive(Clone)]
 pub struct Layout {
     pub terminal_row: u16,
+    pub terminal_column: u16,
     pub name_max_len: usize,
     pub time_start_pos: u16,
 }
@@ -97,6 +109,7 @@ impl Default for State {
             sort_by: config.sort_by,
             layout: Layout {
                 terminal_row: 0,
+                terminal_column: 0,
                 name_max_len: 0,
                 time_start_pos: 0,
             },
@@ -346,7 +359,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Black => {
@@ -355,7 +369,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Blue => {
@@ -364,7 +379,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Cyan => {
@@ -373,7 +389,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Green => {
@@ -382,7 +399,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightBlack => {
@@ -391,7 +409,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightBlue => {
@@ -400,7 +419,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightCyan => {
@@ -409,7 +429,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightGreen => {
@@ -418,7 +439,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightMagenta => {
@@ -427,7 +449,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightRed => {
@@ -436,7 +459,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightWhite => {
@@ -445,7 +469,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::LightYellow => {
@@ -454,7 +479,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Magenta => {
@@ -463,7 +489,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Red => {
@@ -472,7 +499,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Rgb(x, y, z) => {
@@ -481,7 +509,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::White => {
@@ -490,7 +519,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
             Colorname::Yellow => {
@@ -499,7 +529,8 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos
+                    self.layout.time_start_pos,
+                    self.layout.terminal_column
                 );
             }
         }
@@ -577,6 +608,35 @@ impl State {
             }
         }
     }
+
+    pub fn move_cursor(&self, nums: &Num, y: u16) {
+        print!("{}", cursor::Goto(1, self.layout.terminal_row));
+        print!("{}", clear::CurrentLine);
+
+        let item = self.get_item(nums.index);
+        if let Ok(item) = item {
+            match &item.file_ext {
+                Some(ext) => {
+                    print!(
+                        "[{}/{}] {} {}",
+                        nums.index + 1,
+                        self.list.len(),
+                        ext.clone().into_string().unwrap_or_default(),
+                        to_proper_size(item.file_size)
+                    );
+                }
+                None => {
+                    print!(
+                        "[{}/{}] {}",
+                        nums.index + 1,
+                        self.list.len(),
+                        to_proper_size(item.file_size)
+                    );
+                }
+            }
+        }
+        print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
+    }
 }
 
 fn make_item(dir: fs::DirEntry) -> ItemInfo {
@@ -613,10 +673,19 @@ fn make_item(dir: fs::DirEntry) -> ItemInfo {
         .into_string()
         .unwrap_or_else(|_| panic!("failed to get file name."));
 
+    let size = match metadata {
+        Ok(metadata) => metadata.len(),
+        Err(_) => 0,
+    };
+
+    let ext = path.extension().map(|s| s.to_os_string());
+
     ItemInfo {
         file_type: filetype,
         file_name: name,
         file_path: path,
+        file_size: size,
+        file_ext: ext,
         modified: time,
         selected: false,
     }
